-- Version  : $Id: sesam_db_pg.sql,v 1.82 2008/09/03 09:46:26 sesam Exp $
--
-- Title    : Database for SEP sesam
-- File     : sesam_db_pg.sql
-- Author   : P.Schlegel / E.Einwanger  SEP AG D83629 Weyarn
-- Date     : 2007-05-01
-- 
--
-- ########## DROP TABLES ########################

-- ----------------------------------------------------------------------------
-- Tables without active & passive references
-- ----------------------------------------------------------------------------

drop table accounts;
drop table cal_sheets;
drop table defaults;
drop table sessions;
drop table preferences;

-- ----------------------------------------------------------------------------
-- Tables with only active references ( they refer to another table )
-- ----------------------------------------------------------------------------

drop table terms;
drop table capacities;
drop table drive_vendors;
drop table syntaxes;   			
drop table loader_contents;
drop table restore_users;
drop table result_lbls;                 
drop table task_group_relations;        
drop table servers;
drop table user_scheds;

-- ----------------------------------------------------------------------------
-- Tables with active and passive references
-- the sequence of drops MUST be kept due to inner relations
-- ----------------------------------------------------------------------------

drop table task_events;
drop table newday_events;
drop table mediapools_events;
drop table restore_events;
drop table command_events;
drop table tasks;
drop table mediapool_relations;
drop table media;
drop table media_pools;
drop table restore_tasks;
drop table results;
drop table task_sources;        
drop table hw_drives;
drop table hw_loaders;
drop table interfaces;
drop table clients;
drop table locations;

-- ----------------------------------------------------------------------------
-- Tables with only passive references ( they are referred from other tables )
-- ----------------------------------------------------------------------------

drop table commands;
drop table schedules;
drop table drive_types;
drop table media_types;
drop table mediapool_locations;
drop table oper_systems;
drop table drive_groups;
drop table restore_results;
drop table task_groups;
drop table task_types;
drop table params_fix;


-- ########## CREATE TABLES ########################

-- ----------------------------------------------------------------------------
-- Tables with only passive references ( they are referred from other tables )
-- ----------------------------------------------------------------------------


CREATE TABLE drive_types (
	drive_typ       CHAR(15) NOT NULL PRIMARY KEY,
	generic_typ     CHAR(10) NOT NULL 
);

CREATE TABLE media_types (
	media_typ       CHAR(20) NOT NULL PRIMARY KEY
);

CREATE TABLE oper_systems (
        os              CHAR(20) NOT NULL PRIMARY KEY,
        platform        CHAR(20) NOT NULL
); 

CREATE TABLE drive_groups (
	grp_id          INTEGER NOT NULL PRIMARY KEY,
	grp_name        CHAR(50),
	sms_flag        CHAR(1),
        restore_drive   INTEGER,
	text            CHAR(100),
	default_i_name	CHAR(64)
);

CREATE TABLE restore_results (
        restore_id      CHAR(21) NOT NULL PRIMARY KEY,
        start_tim       CHAR(21) NOT NULL,
	r_task		CHAR(49) NOT NULL,
        saveset         CHAR(32) NOT NULL,
        task            CHAR(32) NOT NULL,
        datum           CHAR(21) NOT NULL,
        cnt             INTEGER NOT NULL,
        rest_typ        CHAR(2) NOT NULL,
        genmode         CHAR(2),
        listmode        CHAR(32),
        tree_typ        CHAR(2) NOT NULL,
        mode            CHAR(2) NOT NULL,
        recover         CHAR(64),
        onlinemode      CHAR(64),
        client_id       INTEGER NOT NULL,
        client          CHAR(64) NOT NULL,
        target          CHAR(1024) NOT NULL,
        user_name       CHAR(255) NOT NULL,
	subtask_flag	CHAR(1),
	parent_task	CHAR(49),
        location        CHAR(50),
        drive_num       INTEGER,
        i_name          CHAR(64),
        filter          CHAR(255),
        reloc_source    CHAR(1024),
        state           CHAR(1) NOT NULL,
        stop_tim        CHAR(21),
        size            INTEGER,
        options         CHAR(255),
	throughput      CHAR(64),
        msg             CHAR(255),
	saveset_exist   CHAR(1)
);

CREATE TABLE task_groups (
        grp_name        CHAR(50) NOT NULL PRIMARY KEY,
        text            CHAR(100),
        results_sts     CHAR(1),
        prepost         CHAR(16),
        prepost_switch  CHAR(1)	
);

CREATE TABLE task_types (
        backup_type     CHAR(32) NOT NULL PRIMARY KEY,
        backup_cmd      CHAR(20),
        backup_options  CHAR(128),
        prepost         CHAR(16),
        prepost_cmd     CHAR(20),
        restore_cmd     CHAR(20),
        restore_options CHAR(128),
        r_prepost       CHAR(16),
        r_prepost_cmd   CHAR(20),
        source_prefix   CHAR(20),
        platform        CHAR(20),
        flags           CHAR(32),
        protection      CHAR(4),
        komment         CHAR(40),
        interpreter     CHAR(16)
);

CREATE TABLE params_fix (
	kunde           CHAR(50) NOT NULL PRIMARY KEY,
	rest_eol        INTEGER NOT NULL,
	min_free        INTEGER NOT NULL,
        typ             CHAR(20) NOT NULL,
        version         CHAR(32) NOT NULL,
        inst_date       CHAR(21) NOT NULL,
        inst_cryp       CHAR(21),
        update_date     CHAR(21) NOT NULL,
        val1            CHAR(128),
        val2            CHAR(128),
        val3            CHAR(128),
        val4            CHAR(128),
        serialnumber    CHAR(128),
	sts             CHAR(255),
	msg             CHAR(255)
);

CREATE TABLE mediapool_locations (
    name                CHAR(32) NOT NULL PRIMARY KEY,
    path                CHAR(1024),
    capacity            integer,
    komment             CHAR(80),
    load_cmd            CHAR(255),
    unload_cmd          CHAR(255),
    access_mode         CHAR(255),
    status              CHAR(255),
    device_path         CHAR(255)
);

CREATE TABLE commands (
        name           CHAR(50) NOT NULL PRIMARY KEY,
        owner          CHAR(30) NOT NULL,
        type           CHAR(1) NOT NULL,
        host           CHAR(64) NOT NULL,
        user_name      CHAR(32) NOT NULL,
        command        CHAR(255) NOT NULL
);

-- ----------------------------------------------------------------------------
-- Tables with active and passive references
-- the sequence of CREATE MUST be kept due to inner relations
-- ----------------------------------------------------------------------------

CREATE TABLE schedules (
	name            CHAR(30) NOT NULL PRIMARY KEY,
	cyc_flag        CHAR(1) NOT NULL,
	abs_flag        CHAR(1) NOT NULL,
	p_count         INTEGER,
	p_base          CHAR(3),
	mo              CHAR(1),
	tu              CHAR(1),
	we              CHAR(1),
	th              CHAR(1),
	fr              CHAR(1),
	sa              CHAR(1),
	su              CHAR(1),
	day_of_mth      CHAR(2),
	mth_of_yea      CHAR(2),
	wk_of_mth       CHAR(2),
	day_of_wk       CHAR(10),
	start_date      CHAR(21),
	end_date        CHAR(21),
	exec            CHAR(1),
	start_time      CHAR(5),
	duration        CHAR(5),
	day_offset      INTEGER,
        sepcomment      CHAR(64),
        komment         CHAR(64)
);

CREATE TABLE task_sources (
        task   	        CHAR(32)  NOT NULL,
        version_id      INTEGER      NOT NULL,
        source          CHAR(255) NOT NULL,
        timestamp       CHAR(21),
        user_name       CHAR(255),
        PRIMARY KEY (task,version_id)
);

CREATE TABLE results (
	saveset          CHAR(32) NOT NULL PRIMARY KEY,
        task             CHAR(32) NOT NULL,
	datum            CHAR(21) NOT NULL,
        cnt              INTEGER NOT NULL ,
	fdi_typ          CHAR(2),
	lbl_cnt          INTEGER,
	drive_num        INTEGER,
	client_id        INTEGER,
	client           CHAR(64),
	location         CHAR(50),
	source           CHAR(1024),
        version_id       INTEGER,
	exclude          CHAR(1024),
        priority         INTEGER,
	start_tim        CHAR(21),
	stop_tim         CHAR(21),
        session_id       CHAR(32),
	state            CHAR(1),
	blocks           INTEGER,
	msg              CHAR(254),
        sms_flag         CHAR(1),
        i_name           CHAR(64),
	sbc_start        CHAR(21),
	saveset_exist    CHAR(2),
        media_pool       CHAR(16),
        backup_cmd       CHAR(20),
        backup_type      CHAR(32),
        interpreter      CHAR(32),
        restore_options  CHAR(128),
        bsr_flag         CHAR(1),
        compress_flag    CHAR(1),
        crypt_flag       CHAR(1),
        crypt_key        CHAR(128),
	original_saveset CHAR(64),
	throughput       CHAR(64),
        processed        INTEGER,
        notprocessed     INTEGER, 
        user_comment     CHAR(256)
);

CREATE TABLE restore_tasks (
	r_task		CHAR(49) NOT NULL PRIMARY KEY,
        saveset         CHAR(32) NOT NULL,
        task            CHAR(32) NOT NULL,
        datum           CHAR(21) NOT NULL,
        cnt             INTEGER NOT NULL,
        rest_typ        CHAR(2) NOT NULL,
        genmode         CHAR(2),
        listmode        CHAR(32),
        tree_typ        CHAR(2) NOT NULL,
        mode            CHAR(2) NOT NULL,
        overwrite       CHAR(8),
        original        CHAR(2),
        recover         CHAR(64),
        onlinemode      CHAR(64),
        client_id       INTEGER NOT NULL,
        client          CHAR(64) NOT NULL,
        target          CHAR(1024) NOT NULL,
        user_name       CHAR(255) NOT NULL,
	subtask_flag	CHAR(1),
	parent_task	CHAR(49),
        location        CHAR(50),
        drive_num       INTEGER,
        i_name          CHAR(64),
        filter          CHAR(255),
        reloc_source    CHAR(1024),
        r_prepost       CHAR(16),
	make_stamp	CHAR(21) NOT NULL,
        eol             INTEGER,
        restore_cmd     CHAR(64),
        options         CHAR(128),
	komment		CHAR(255)
);

CREATE TABLE locations (
        id              INTEGER NOT NULL PRIMARY KEY,
	name            CHAR(50),
	parent          INTEGER references locations (id),
	describe        CHAR(128),
	contact         CHAR(128),
	komment         CHAR(128)
);

CREATE TABLE clients (
	client_id       INTEGER NOT NULL PRIMARY KEY,
        name            CHAR(64) NOT NULL,
	location        INTEGER references locations (id),
	os              CHAR(20) NOT NULL references oper_systems (os) DEFERRABLE INITIALLY DEFERRED,
	net_prot        CHAR(10) NOT NULL ,
	accessmode      CHAR(10),
	komment         CHAR(255),
	user_comment    CHAR(255),
   	permit          CHAR(1),
        user_name       CHAR(255),
        password        CHAR(255),
	access_options  CHAR(255),
	stpd_options    CHAR(32),
	stpd_port       CHAR(32),
        sbc_version     CHAR(20),
        data_mover      CHAR(64)
);

CREATE TABLE hw_loaders (
	loader_num      INTEGER NOT NULL PRIMARY KEY,
	device          CHAR(254) NOT NULL,
	client_id       INTEGER NOT NULL references clients (client_id) DEFERRABLE INITIALLY DEFERRED,
	name            CHAR(32),
	typ             CHAR(32) NOT NULL,
	ctrl            CHAR(10) NOT NULL,
	slots           INTEGER NOT NULL,
        drives          INTEGER,
        ports           INTEGER,
        transport       INTEGER,
        first_slot      INTEGER,
        last_slot       INTEGER,
        autounload      CHAR(1),
        barcode         CHAR(1)
);

CREATE TABLE hw_drives (
	drive_num       INTEGER NOT NULL PRIMARY KEY,
	device          CHAR(254) NOT NULL ,
	client_id       INTEGER NOT NULL references clients (client_id) DEFERRABLE INITIALLY DEFERRED,
	typ             CHAR(15) NOT NULL references drive_types (drive_typ) DEFERRABLE INITIALLY DEFERRED,
        name            CHAR(32),
	loader_num      INTEGER references hw_loaders (loader_num) DEFERRABLE INITIALLY DEFERRED,
	kompress        CHAR(1),
	sts             CHAR(132),
	msg             CHAR(132),
	occupy          CHAR(20),
	done            INTEGER,
	run             INTEGER,
	waiting         INTEGER,
       	throughput      CHAR(32),
        grp_id          INTEGER references drive_groups (grp_id) DEFERRABLE INITIALLY DEFERRED,
        sms_cnts        INTEGER,
       	label           CHAR(30),
        timeout         INTEGER,
        media_timeout   INTEGER,
        clean_bit       CHAR(1),
        loader_drive    INTEGER,
        path            CHAR(132),
        block_size      INTEGER,
        sms_nr          INTEGER,
        segment_size    INTEGER
);


CREATE TABLE media_pools (
        name             CHAR(16) NOT NULL PRIMARY KEY,
        pool_id          INTEGER NOT NULL,
        descript         CHAR(80),
        drive_grp        INTEGER NOT NULL references drive_groups (grp_id) DEFERRABLE INITIALLY DEFERRED,
        media_strg       CHAR(64) NOT NULL,
        media_chg        CHAR(8) NOT NULL,
        disk_capac       CHAR(16),
        disk_dir         CHAR(254),
        eol              INTEGER,
        accept_empty     CHAR(2),
        accept_eol       CHAR(2),
        accept_spare     CHAR(2),
        use_storagepools CHAR(2)

);

CREATE TABLE mediapool_relations (
    mediapool           CHAR(16) references media_pools (name) DEFERRABLE INITIALLY DEFERRED,
    mediapool_location  CHAR(32) references mediapool_locations (name) DEFERRABLE INITIALLY DEFERRED,
    priority            INTEGER,
    komment             CHAR(80)
);


CREATE TABLE interfaces (
	client_id       INTEGER  NOT NULL references clients (client_id) DEFERRABLE INITIALLY DEFERRED,
        i_name          CHAR(64) NOT NULL UNIQUE,
        PRIMARY KEY (client_id,i_name)
);


-- task_events.object references tasks.task or task_groups.grp_name

CREATE TABLE task_events (
        id              INTEGER NOT NULL PRIMARY KEY,
        object          CHAR(50) NOT NULL,
        schedule        CHAR(30), 
        grp_flag        CHAR(1) NOT NULL,
        priority        INTEGER NOT NULL,
        suppress        CHAR(1),
        owner           CHAR(30) NOT NULL,
        fdi_typ         CHAR(2) NOT NULL,
        media_pool      CHAR(16) NOT NULL,
        label           CHAR(30),
        drive_num       INTEGER references hw_drives (drive_num) DEFERRABLE INITIALLY DEFERRED,
	i_name		CHAR(64) references interfaces (i_name) DEFERRABLE INITIALLY DEFERRED,
        data_mover      CHAR(64),
        max_since_full  CHAR(21)
);

CREATE TABLE newday_events (
        id              INTEGER NOT NULL PRIMARY KEY,
        object          CHAR(50) NOT NULL references params_fix (kunde) DEFERRABLE INITIALLY DEFERRED,
        schedule        CHAR(30),
        priority        INTEGER NOT NULL,
        suppress        CHAR(1),
        owner           CHAR(30) NOT NULL
);

CREATE TABLE mediapools_events (
        id              INTEGER NOT NULL PRIMARY KEY,
        object          CHAR(16) NOT NULL references media_pools (name) DEFERRABLE INITIALLY DEFERRED,
        drive           CHAR(16),
        schedule        CHAR(30), 
        priority        INTEGER NOT NULL,
        suppress        CHAR(1),
        owner           CHAR(30) NOT NULL,
        grp_flag        CHAR(1) NOT NULL,
        m_count         INTEGER,
        label           CHAR(30),
        emergency       CHAR(30)
);

CREATE TABLE restore_events (
        id               INTEGER NOT NULL PRIMARY KEY,
	object           CHAR(49) NOT NULL references restore_tasks (r_task) DEFERRABLE INITIALLY DEFERRED,
        schedule         CHAR(30),
        priority         INTEGER NOT NULL,
        suppress         CHAR(1),
        owner            CHAR(30) NOT NULL,
        drive_num        INTEGER,
        i_name           CHAR(64),
        listmode         CHAR(32),
        absolute_flag    CHAR(1),
        saveset_start    INTEGER,
        saveset_end      INTEGER,
        min_max          CHAR(21),
        date_start       CHAR(21),
        date_end         CHAR(21),
        state            CHAR(12),
        cfdi_type        CHAR(12),
        media_pools      CHAR(255),
        media_tolerance  CHAR(21),
        media_preference CHAR(21),
        data_mover       CHAR(64),
        use_saveset      CHAR(32),
        restore_cmd      CHAR(64),
        options          CHAR(128)

);

CREATE TABLE command_events (
        id               INTEGER NOT NULL PRIMARY KEY,
        object           CHAR(50) NOT NULL,
        schedule         CHAR(30),
        priority         INTEGER NOT NULL,
        suppress         CHAR(1),
        owner            CHAR(30) NOT NULL,
        type             CHAR(1) NOT NULL,
        host             CHAR(64) NOT NULL,
        user_name        CHAR(32) NOT NULL,
        command          CHAR(255) NOT NULL
);

CREATE TABLE media (
    	label           CHAR(30) NOT NULL PRIMARY KEY,
    	barcode         CHAR(30) ,
        pool            CHAR(16) NOT NULL references media_pools (name) DEFERRABLE INITIALLY DEFERRED,
        id              INTEGER NOT NULL ,
	sperrkz         CHAR(1) NOT NULL ,
	sichtag         CHAR(21) NOT NULL ,
	eol             CHAR(21),
	drive_num       INTEGER,
        init_drive      INTEGER,
	folgekass       INTEGER,
	kasstyp         CHAR(20) NOT NULL references media_types (media_typ) DEFERRABLE INITIALLY DEFERRED,
	lader           INTEGER,
	slot            INTEGER,
	gefuellt        INTEGER,
	eom_state       CHAR(1),
	first_init      CHAR(21),
        close_tape      CHAR(1),
	init_flag       CHAR(1),
	error           INTEGER,
	error_delt      INTEGER,
	use_count       INTEGER,
	location        CHAR(32),
	duplflag        CHAR(1),
	kommentar       CHAR(254),
	user_comment    CHAR(254)
);

CREATE TABLE tasks (
	task   	        CHAR(32) NOT NULL PRIMARY KEY,
        backup_type     CHAR(32) references task_types (backup_type) DEFERRABLE INITIALLY DEFERRED,
	client_id       INTEGER NOT NULL references clients (client_id) DEFERRABLE INITIALLY DEFERRED,
	source          CHAR(1024),
	exclude         CHAR(1024),
	komment         CHAR(255),		
	results_sts     CHAR(1),
        backup_cmd      CHAR(64),
	backup_options  CHAR(255),
	restore_options CHAR(255),
	prepost         CHAR(16),
        r_prepost       CHAR(16),
        nfs_mount       CHAR(1),
        filesystem      CHAR(1),
        bsr_flag        CHAR(1),
        compress_flag   CHAR(1),
        crypt_flag      CHAR(1),
        crypt_key       CHAR(128)
);

-- ----------------------------------------------------------------------------
-- Tables with only active references ( they refer to another table )
-- ----------------------------------------------------------------------------

CREATE TABLE terms (
       id               INTEGER NOT NULL PRIMARY KEY,
       type             CHAR(1) NOT NULL,
       schedule         CHAR(30) NOT NULL references schedules (name) DEFERRABLE INITIALLY DEFERRED,
       next_exec        CHAR(21),
       next_end         CHAR(21)
);

CREATE TABLE capacities (
	drive_typ       CHAR(15) NOT NULL references drive_types (drive_typ) DEFERRABLE INITIALLY DEFERRED,
	media_typ       CHAR(20) NOT NULL references media_types (media_typ) DEFERRABLE INITIALLY DEFERRED,
	kapaz_num       INTEGER NOT NULL,
        protection      CHAR(4) NOT NULL,
        PRIMARY KEY (drive_typ,media_typ)
);

CREATE TABLE drive_vendors (
	drive_typ       CHAR(15) NOT NULL references drive_types (drive_typ) DEFERRABLE INITIALLY DEFERRED,
	vendor_typ      CHAR(16) NOT NULL,
        protection      CHAR(4) NOT NULL,
	PRIMARY KEY (drive_typ,vendor_typ)
);

CREATE TABLE syntaxes (
	os              CHAR(20) NOT NULL references oper_systems (os) DEFERRABLE INITIALLY DEFERRED,
	command         CHAR(20) NOT NULL,
	syntax          CHAR(50) NOT NULL,
        PRIMARY KEY (os,command)
);

CREATE TABLE task_group_relations (
        grp_name        CHAR(50) NOT NULL references task_groups (grp_name) DEFERRABLE INITIALLY DEFERRED,
	task   	        CHAR(32) NOT NULL references tasks (task) DEFERRABLE INITIALLY DEFERRED,
	task_order      INTEGER,
        PRIMARY KEY (grp_name,task)
);

CREATE TABLE result_lbls (
        saveset         CHAR(32) NOT NULL references results (saveset) DEFERRABLE INITIALLY DEFERRED,
	num             INTEGER NOT NULL,
	label           CHAR(30) NOT NULL,
        segment         CHAR(20) NOT NULL,
        PRIMARY KEY (saveset,num)
);

CREATE TABLE restore_users (
	name            CHAR(255) NOT NULL,
	task            CHAR(32) NOT NULL references tasks (task) DEFERRABLE INITIALLY DEFERRED,
	path            CHAR(132) NOT NULL,
	komment         CHAR(100),
        PRIMARY KEY (name,task,path)
);

CREATE TABLE loader_contents (
        loader          INTEGER	NOT NULL references hw_loaders (loader_num) DEFERRABLE INITIALLY DEFERRED,
        object          CHAR(10) NOT NULL,
        number          INTEGER	NOT NULL,
        lbl             CHAR(30),
	barcode         CHAR(30),
        check_lbl       CHAR(1),
        checkedlbl      CHAR(30),
        from_object     CHAR(10),
        from_number     INTEGER,
        to_object       CHAR(10),
        to_number       INTEGER,	
        PRIMARY KEY (loader,object,number)
);

CREATE TABLE servers (
        name            CHAR(64) NOT NULL PRIMARY KEY,
	os              CHAR(20) NOT NULL references oper_systems (os) DEFERRABLE INITIALLY DEFERRED,
        ip_address      CHAR(64),
        rmi_port        INTEGER,
        ctrl_port       INTEGER,
        db_port         INTEGER,
        db_name         CHAR(73),
        db_alias        CHAR(73),
        db_type         CHAR(32),
        connect         CHAR(1),
        komment         CHAR(80),
        id              INTEGER
);

CREATE TABLE user_scheds (
     name            CHAR(30) NOT NULL references schedules (name) DEFERRABLE INITIALLY DEFERRED,
     date_name       CHAR(32) NOT NULL,
     date_pattern    CHAR(32) NOT NULL,
     next_exec       CHAR(32),
     notices         CHAR(100),
     PRIMARY KEY (name,date_name)
);

-- ----------------------------------------------------------------------------
-- Tables without active & passive references
-- ----------------------------------------------------------------------------

CREATE TABLE accounts (
        account_name       CHAR(64) NOT NULL PRIMARY KEY,
        kunde              CHAR(64),
        smtp_email_address CHAR(64) NOT NULL,
        smtp_server        CHAR(64) NOT NULL,
        mail_username      CHAR(64),
        mail_password      CHAR(128),
        mail_to            CHAR(128) NOT NULL,
        mail_cc            CHAR(128),
        mail_bcc           CHAR(128),
        service_location   CHAR(64)
);

CREATE TABLE cal_sheets (
	id		CHAR(30) PRIMARY KEY,
	start		CHAR(21),
	end_time        CHAR(21),
	ev_type		CHAR(3),
	ev_id		INTEGER,
	schedule	CHAR(30),
        priority        INTEGER,
	state		CHAR(3),
        object          CHAR(132),
        suppress        CHAR(1),
	msg		CHAR(132)
);

CREATE TABLE defaults (	
        key             CHAR(32) NOT NULL,
        user_name       CHAR(255) NOT NULL ,
        value           CHAR(255) NOT NULL,
        host            CHAR(255),
        protection      CHAR(4),
        PRIMARY KEY     (key,user_name)
);


CREATE TABLE sessions (
	session_id      CHAR(32) PRIMARY KEY,
	object		CHAR(32),
	komment		CHAR(80)
);


CREATE TABLE preferences (
        pref_node       CHAR(255)  NOT NULL,
        pref_user       CHAR(255)  NOT NULL,
        pref_key        CHAR(255)  NOT NULL,
        pref_value      CHAR(255)  NOT NULL,
        PRIMARY KEY     (pref_node,pref_user,pref_key)
);
